@@ -96,15 +96,8 @@ package Sidef::Types::Array::Pair {
 
             my $s;
 
-            "Pair(" . "\n" . join(
-                ",\n",
-
-                (' ' x ($Sidef::SPACES += $Sidef::SPACES_INCR))
-                  . join(", ",
-                         map { (ref($_) && ($s = UNIVERSAL::can($_, 'dump'))) ? $s->($_) : defined($_) ? $_ : 'nil' } @$obj)
-              )
-              . "\n"
-              . (' ' x ($Sidef::SPACES -= $Sidef::SPACES_INCR)) . ")";
+            'Pair('
+              . join(', ', map { (ref($_) && ($s = UNIVERSAL::can($_, 'dump'))) ? $s->($_) : ($_ // 'nil') } @$obj) . ')';
         };
 
         local *Sidef::Types::Array::Pair::dump = $sub;
@@ -114,6 +107,9 @@ package Sidef::Types::Array::Pair {
     sub dump {
         Sidef::Types::String::String->new($_[0]->_dump);
     }
+
+    *to_s   = \&dump;
+    *to_str = \&dump;
 };
 
 1
