@@ -32,6 +32,7 @@ package Sidef::Types::Bool::Bool {
 
     sub get_value { ${$_[0]} }
     sub to_bool   { $_[0] }
+
     *to_b = \&to_bool;
 
     {
@@ -69,6 +70,9 @@ package Sidef::Types::Bool::Bool {
         Sidef::Types::String::String->new($$self ? 'true' : 'false');
     }
 
+    *to_s   = \&dump;
+    *to_str = \&dump;
+
 };
 
 1
