@@ -306,7 +306,12 @@ package Sidef::Types::Hash::Hash {
 
     sub concat {
         my ($self, $obj) = @_;
-        bless {%$self, %$obj}, ref($self);
+
+#<<<
+            UNIVERSAL::isa($obj, __PACKAGE__)                  ? bless({%$self, %$obj}, ref($self))
+          : UNIVERSAL::isa($obj, 'Sidef::Types::Array::Array') ? bless({%$self, @$obj}, ref($self))
+          :                                                      bless({%$self,  $obj}, ref($self));
+#>>>
     }
 
     *merge = \&concat;
@@ -589,9 +594,10 @@ package Sidef::Types::Hash::Hash {
         *{__PACKAGE__ . '::' . '+'}   = \&concat;
         *{__PACKAGE__ . '::' . '=='}  = \&eq;
         *{__PACKAGE__ . '::' . '!='}  = \&ne;
-        *{__PACKAGE__ . '::' . '≠'} = \&ne;
-        *{__PACKAGE__ . '::' . ':'}   = \&new;
+        *{__PACKAGE__ . '::' . '≠'}   = \&ne;
         *{__PACKAGE__ . '::' . '...'} = \&to_list;
+
+        #*{__PACKAGE__ . '::' . ':'}   = \&new;#
     }
 };
 
