@@ -133,6 +133,9 @@ package Sidef::Types::Range::RangeString {
         my ($self) = @_;
         Sidef::Types::String::String->new("$self");
     }
+
+    *to_s   = \&dump;
+    *to_str = \&dump;
 }
 
 1;
